/***************************************************************************
                          oooqs.cpp  -  description
                             -------------------
    begin                : Mon Apr 29 17:48:15 CEST 2002
    copyright            : (C) 2002 by Christian Nitschkowski
    email                : segfault_ii@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "oooqs.h"
#include <prefsdialog.h>

#include <qcursor.h>
#include <qfile.h>
#include <qiodevice.h>
#include <qtooltip.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>

#define Icon(x) KGlobal::instance()->iconLoader()->loadIcon(x, KIcon::Toolbar)

OooQs::OooQs(KAboutData* aboutDta, KDialog* d, QWidget *parent, const char *name) : KSystemTray(parent, name), aboutDialog(d)
{
	QString filter;

	/** Make sure, we won't access a non-existing object */
	open = 0;
	actionCollection = new KActionCollection(this);
	popupMenu = new KPopupMenu(this);

	aboutData = aboutDta;

	/** Set the OpenOffice icon */
	setPixmap(Icon("oooqs"));

	/** Create the preferences-dialog */
	preferencesDialog = new prefsDialog(this);
	startDialog = new openDialog(this);

	help = new KHelpMenu( this, 0, false );

	/** Define the actions */
	quit = KStdAction::quit(kapp, SLOT (quit()), actionCollection);
	file = KStdAction::open(this, SLOT (fileOpen()), actionCollection);
	recent = KStdAction::openRecent(this, SLOT (openRecent(const KURL&)), actionCollection);
	preferences = KStdAction::preferences (this, SLOT (openPreferences()), actionCollection);
	start = new KAction(i18n("Start with..."), Icon("launch"), 0, 0, 0, actionCollection, 0);
	calc = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	draw = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	impress = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	global = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	math = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	web = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	writer = new KAction("", 0, 0, 0, 0, actionCollection, 0);
	admin = new KAction("", 0, 0, 0, 0, actionCollection, 0);

	/** Connect the actions with the slots */
	connect (calc, SIGNAL (activated()), SLOT (startCalc()));
	connect (draw, SIGNAL (activated()), SLOT (startDraw()));
	connect (impress, SIGNAL (activated()), SLOT (startImpress()));
	connect (global, SIGNAL (activated()), SLOT (startGlobal()));
	connect (math, SIGNAL (activated()), SLOT (startMath()));
	connect (web, SIGNAL (activated()), SLOT (startWeb()));
	connect (writer, SIGNAL (activated()), SLOT (startWriter()));
	connect (preferencesDialog, SIGNAL (success()), SLOT (changedConfig()));
	connect (preferencesDialog->clear, SIGNAL (clicked()), this, SLOT (clearURLList()));
	connect (start, SIGNAL (activated()), SLOT(openStart()));
	connect (startDialog, SIGNAL (selected(QStringList)), this, SLOT (startInstances(QStringList)));
	connect (help, SIGNAL (showAboutApplication()), this, SLOT (showAbout()));
	connect (admin, SIGNAL (activated()), this, SLOT (startSPAdmin()));

	/** Read the configuration */
	config = KGlobal::config();

	/** Load the list of recent files */
	recent->loadEntries(config);

	/** Try to find the entries in the configfile */
	/** If the config is not accessible (e.g. not existing) */
	config->setGroup("General options");
	quickstartEnabled = config->readBoolEntry("quickstart", false);
	if (config->readBoolEntry("first", true)){
		/** Fill the variables with default values */
		rereadConfig();
		/** Display a messagebox and execute the preferencesDialog */
		KMessageBox::information(0, i18n(\
			"You don't seem to have a valid configuration.\
That's ok if you start OpenOffice.org Quickstarter for the first time.\n\n\
The configuration-dialog will now be called so that you can enter the required data.\n\
You can access the dialog any time later by selecting \"Configure OpenOffice.org Quickstarter...\" in the context-menu."),\
i18n("No valid config found"));
		/*if (preferencesDialog->exec() == QDialog::Rejected){
			KMessageBox::information(this, "wir sollten uns eigentlich beenden");
			//kapp->exit(0);
		}*/
		preferencesDialog->exec();
	}else{
		rereadConfig();
	}

	/** Create the fileOpen-dialog */
	filter="*.sxc *.sxd *.sxi *.sxg *.sxm *.sxw|" + suite + " " + i18n("document") + "\n";
	filter+="*.sxc|" + suite + " " + i18n("Calc") + " " + i18n("document") + "\n";
	filter+="*.sxd|" + suite + " " + i18n("Draw") + " " + i18n("document") + "\n";
	filter+="*.sxi|" + suite + " " + i18n("Impress") + " " + i18n("document") + "\n";
	filter+="*.sxg|" + suite + " " + i18n("Global") + " " + i18n("document") + "\n";
	filter+="*.sxm|" + suite + " " + i18n("Math") + " " + i18n("document") + "\n";
	filter+="*.sxw|" + suite + " " + i18n("Writer") + " " + i18n("document") + "\n";
	filter+="*.sda *.sdc *.sgl *.sdw|" + i18n("Star Office 5.x") + " " + i18n("document") + "\n";
	filter+="*.sdc|" + i18n("Star Office 5.x") + " " + i18n("StarCalc") + " " + i18n("document") + "\n";
	filter+="*.sda|" + i18n("Star Office 5.x") + " " + i18n("StarDraw or StarImpress") + " " + i18n("document") + "\n";
	filter+="*.sgl|" + i18n("Star Office 5.x") + " " + i18n("StarWriter") + " " + i18n("Globaldocument") + "\n";
	filter+="*.sdw|" + i18n("Star Office 5.x") + " " + i18n("StarWriter") + " " + i18n("document") + "\n";
	filter+="*.html|" + i18n("HTML") + " " + i18n("document") + "\n";
	filter+="*.doc|" + i18n("Microsoft Word") + " " + i18n("document") + "\n";
	filter+="*.xls|" + i18n("Microsoft Excel") + " " + i18n("document") + "\n";
	filter+="*.ppt|" + i18n("Microsoft PowerPoint") + " " + i18n("document") + "\n";
	filter+="*.txt|" + i18n("Simple ASCII-text file") + "\n";
	filter+="*.csv|" + i18n("Simple Comma seperated table") + "\n";
	filter+="*.sxc *.sxd *.sxi *.sxg *.sxm *.sxw *.sda *.sdc *.sgl *.sdw *.html *.doc *.xls *.ppt *.txt *.csv|" + i18n("All compatible documents") + "\n";
	filter+="*|" + i18n("All files");

	open = new KFileDialog(config->readEntry("documents"), filter, this, 0, true);
	open->setMode(4+8+16);

	instance << soffice << "-quickstart";

	/** Check if the quickstart should be enabled */
	if (quickstartEnabled){
		/** Make sure that instance is automatically restarted when it gets closed */
		connect(&instance, SIGNAL (processExited(KProcess*)), SLOT (restartOffice()));
		/** Try to start instance for the first time */
		restartOffice();
	}
}

OooQs::~OooQs()
{
}

/** Opens a KFileDialog that let's you choose some files you want to open. */
void OooQs::fileOpen(){
	QStringList fileList;
	QString last;

	if (open->exec()){
		fileList = open->selectedFiles();
		for ( QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it ) {
			last = *it;
			recent->addURL(*it);
		}
		config->setGroup("General options");
		config->writeEntry("Last file", fileList.last());
		/** the exit Handler only works, if the father ist notified */
		recent->saveEntries(config);
		config->sync();
		startInstances(fileList);
	}
}
/** Opens the selected file. */
void OooQs::openRecent(const KURL& url){
	config->setGroup("General options");
	config->writeEntry("Last file", url.url());
	startInstances(url.url());
}
/** Executes the about-dialog. */
void OooQs::showAbout(){
	aboutDialog->exec();
}
/** Starts OpenOffice.org Calc with a new file */
void OooQs::startCalc(){
	startInstances("private:factory/scalc");
}
/** Starts OpenOffice.org Draw with a new file */
void OooQs::startDraw(){
	startInstances("private:factory/sdraw");
}
/** Starts OpenOffice.org Impress with a new file */
void OooQs::startImpress(){
	startInstances("private:factory/simpress");
}
/** Starts OpenOffice.org Global with a new file */
void OooQs::startGlobal(){
	startInstances("private:factory/swriter/GlobalDocument");
}
/** Starts OpenOffice.org Math with a new file */
void OooQs::startMath(){
	startInstances("private:factory/smath");
}
/** Starts OpenOffice.org Writer with a new file */
void OooQs::startWriter(){
	startInstances("private:factory/swriter");
}
/** Starts OpenOffice.org Web with a new file */
void OooQs::startWeb(){
	startInstances("private:factory/swriter/web");
}

/** Restarts OpenOffice if removed from memory. */
void OooQs::restartOffice(){
	/** at first check if the quickstart-mode is enabled */
	if (quickstartEnabled){
		/** check, if the Master Instance is running ! */
		if (!instance.isRunning()) {
			/** we dont need any Communication ! */
			instance.start(KProcess::NotifyOnExit,KProcess::NoCommunication);
		}
	}
}

/** Deletes the process. */
void OooQs::delProcess(KProcess* process){
	delete process;
}
/** Call this to open the start-dialog. */
void OooQs::openStart(){
	startDialog->exec();
}
/** Call this to open the preferences-dialog. */
void OooQs::openPreferences(){
	preferencesDialog->exec();
}
/** Should be called whenever the configuration changes. */
void OooQs::changedConfig(){
	config->setGroup("General options");
	config->writeEntry("first", false);
	config->writeEntry("exec", preferencesDialog->binary->url());
	config->writeEntry("quickstart", preferencesDialog->enableQuick->isChecked());
	config->writeEntry("documents", preferencesDialog->documents->url());
	config->writeEntry("templates", preferencesDialog->templates->url());
	config->writeEntry("maxRecentFiles", preferencesDialog->maxEntries->value());
	config->writeEntry("suite", preferencesDialog->ooffice->isChecked());
	config->writeEntry("detailed", preferencesDialog->detailedMenus->isChecked());
	config->writeEntry("save", preferencesDialog->saveSelections->isChecked());
	config->writeEntry("quickFix", preferencesDialog->quickStartFix->isChecked());
	config->setGroup("Modules");
	config->writeEntry("calc", preferencesDialog->calc->isChecked());
	config->writeEntry("draw", preferencesDialog->draw->isChecked());
	config->writeEntry("impress", preferencesDialog->impress->isChecked());
	config->writeEntry("global", preferencesDialog->global->isChecked());
	config->writeEntry("math", preferencesDialog->math->isChecked());
	config->writeEntry("web", preferencesDialog->web->isChecked());
	config->writeEntry("writer", preferencesDialog->writer->isChecked());
	config->writeEntry("open", preferencesDialog->open->isChecked());
	config->writeEntry("recent", preferencesDialog->recent->isChecked());
	config->setGroup("Additional");
	config->writeEntry("stdCalc", preferencesDialog->stdCalc->isChecked());
	config->writeEntry("stdDraw", preferencesDialog->stdDraw->isChecked());
	config->writeEntry("stdImpress", preferencesDialog->stdImpress->isChecked());
	config->writeEntry("stdGlobal", preferencesDialog->stdGlobal->isChecked());
	config->writeEntry("stdMath", preferencesDialog->stdMath->isChecked());
	config->writeEntry("stdWeb", preferencesDialog->stdWeb->isChecked());
	config->writeEntry("stdWriter", preferencesDialog->stdWriter->isChecked());
	config->writeEntry("stdOpen", preferencesDialog->stdOpen->isChecked());
	config->writeEntry("stdLast", preferencesDialog->stdLast->isChecked());
	config->writeEntry("stdStart", preferencesDialog->stdStart->isChecked());
	config->writeEntry("nothing", preferencesDialog->nothing->isChecked());
	config->sync();
	rereadConfig();
}
/** Call this to (re-)load the config from disk */
void OooQs::rereadConfig(){
	KConfig* kickerconf = new KConfig("kickerrc", true);
	kickerconf->setGroup("menus");
	QString tempURL = "";
	/** Reparse the configuration to be shure that the data we use is up to date */
	config->reparseConfiguration();
	/** Set the values */
	config->setGroup("General options");
	soffice = config->readEntry("exec", QDir::homeDirPath() + "/OpenOffice.org1.0");
	sofficeFound=false;
	if (QFile::exists(soffice+"/soffice")){
		soffice+="/soffice";
		sofficeFound=true;
	}
	else if (QFile::exists(soffice+"/program/soffice")){
		soffice+="/program/soffice";
		sofficeFound=true;
	}
	else if (QFile::exists(soffice+"/program/soffice.bin")){
		soffice+="/program/soffice.bin";
		sofficeFound=true;
	}
	if (quickstartEnabled != config->readBoolEntry("quickstart", false)){
		if (quickstartEnabled){
			quickstartEnabled=false;
			if (instance.isRunning()){
				if (KMessageBox::questionYesNo(0, i18n("There is still a quickstart-instance of %1 running.\nDo you want to kill it now?\n\nMake sure you close all your %2 documents before answering yes!").arg(suite).arg(suite), i18n("Running instance of %1 detected").arg(suite)) == 3){
					instance.kill();
				}
			}
		}else{
			quickstartEnabled=true;
			restartOffice();
		}
	}
	if (open){
		tempURL = config->readEntry("documents", "not_there");
		if (tempURL != "not_there"){
			open->setURL(tempURL);
		}
	}
	recent->setMaxItems(config->readUnsignedNumEntry("maxRecentFiles", 5));
	tempURL = config->readEntry("templates", "not_there");
	if (tempURL != "not_there"){
		startDialog->templateView->setURL(tempURL, true);
	}

	/** Delete the menu */
	popupMenu->clear();

	/** Setup the labelstrings which are shown on the actions and the newDocView-entries */
	setupLabels();

	int counter = 1;
	int modules = 0;
	popupMenu->insertTitle(Icon("oooqs"), aboutData->programName(), -1, counter);
	counter++;
	config->setGroup("Modules");
	if (sofficeFound){
		if (config->readBoolEntry("open", sofficeFound)){
			file->plug(popupMenu, counter);
			counter++;
			if (config->readBoolEntry("recent", sofficeFound)){
				recent->plug(popupMenu, counter);
				counter++;
			}
			popupMenu->insertSeparator(counter);
			counter++;
		}
		if (config->readBoolEntry("calc", sofficeFound)){
			calc->plug(popupMenu, counter);
			modules|=Calc;
			counter++;
		}
		if (config->readBoolEntry("draw", sofficeFound)){
			draw->plug(popupMenu, counter);
			modules|=Draw;
			counter++;
		}
		if (config->readBoolEntry("impress", sofficeFound)){
			impress->plug(popupMenu, counter);
			modules|=Impress;
			counter++;
		}
		if (config->readBoolEntry("global", sofficeFound)){
			global->plug(popupMenu, counter);
			modules|=Global;
			counter++;
		}
		if (config->readBoolEntry("math", sofficeFound)){
			math->plug(popupMenu, counter);
			modules|=Math;
			counter++;
		}
		if (config->readBoolEntry("web", sofficeFound)){
			web->plug(popupMenu, counter);
			modules|=Web;
			counter++;
		}
		if (config->readBoolEntry("writer", sofficeFound)){
			writer->plug(popupMenu, counter++);
			modules|=Writer;
			counter++;
		}
		if (modules){
			popupMenu->insertSeparator(counter);
			counter++;
		}
		start->plug(popupMenu, counter);
		counter++;
		popupMenu->insertSeparator(counter);
		counter++;
	}
	preferences->plug(popupMenu, counter);
	counter++;
	if (sofficeFound){
		admin->plug(popupMenu, counter);
		counter++;
	}
	popupMenu->insertSeparator(counter);
	counter++;
	popupMenu->insertItem(i18n("&Help"), help->menu() );
	popupMenu->insertSeparator(counter);
	counter++;
	quit->plug(popupMenu, counter);
	popupMenu->insertTearOffHandle(-1, -1);

	calc->setEnabled(config->readBoolEntry("calc", sofficeFound));
	draw->setEnabled(config->readBoolEntry("draw", sofficeFound));
	impress->setEnabled(config->readBoolEntry("impress", sofficeFound));
	global->setEnabled(config->readBoolEntry("global", sofficeFound));
	math->setEnabled(config->readBoolEntry("math", sofficeFound));
	web->setEnabled(config->readBoolEntry("web", sofficeFound));
	writer->setEnabled(config->readBoolEntry("writer", sofficeFound));
	file->setEnabled(config->readBoolEntry("open", sofficeFound));
	if (file->isEnabled()){
		recent->setEnabled(config->readBoolEntry("recent", sofficeFound));
	}else{
		recent->setEnabled(false);
	}

	/** Setup the start-with-dialog */
	startDialog->setup_newDocView(modules);
	if (writer->isEnabled()){
		startDialog->setup_wizardView(AllW);
	}else{
		startDialog->setup_wizardView(NoneW);
	}
	startDialog->setup_templateView();
	startDialog->checkStates();

	setDoubleClickAction();

	/** Set the widgets in the preferences dialog to the current values */
	/** The additional-group has to be loaded before the modules.
		If not, it could happen that disabled radiobuttons are checked */
	config->setGroup("General options");
	preferencesDialog->binary->setURL(config->readEntry("exec", QDir::homeDirPath() + "/OpenOffice.org1.0"));
	preferencesDialog->enableQuick->setChecked(quickstartEnabled);
	preferencesDialog->maxEntries->setValue(recent->maxItems());
	preferencesDialog->documents->setURL(config->readEntry("documents", QDir::homeDirPath()));
	preferencesDialog->templates->setURL(config->readEntry("templates", QDir::homeDirPath()));
	preferencesDialog->ooffice->setChecked(config->readBoolEntry("suite", true));
	preferencesDialog->soffice->setChecked(!(config->readBoolEntry("suite", true)));
	preferencesDialog->detailedMenus->setChecked(config->readBoolEntry("detailed", kickerconf->readBoolEntry("DetailedMenuEntries")));
	preferencesDialog->saveSelections->setChecked(config->readBoolEntry("save", true));
	preferencesDialog->quickStartFix->setEnabled(quickstartEnabled);
	preferencesDialog->quickStartFix->setChecked(config->readBoolEntry("quickFix", false));
	config->setGroup("Additional");
	preferencesDialog->stdCalc->setChecked(config->readBoolEntry("stdCalc", false));
	preferencesDialog->stdDraw->setChecked(config->readBoolEntry("stdDraw", false));
	preferencesDialog->stdImpress->setChecked(config->readBoolEntry("stdImpress", false));
	preferencesDialog->stdGlobal->setChecked(config->readBoolEntry("stdGlobal", false));
	preferencesDialog->stdMath->setChecked(config->readBoolEntry("stdMath", false));
	preferencesDialog->stdWeb->setChecked(config->readBoolEntry("stdWeb", false));
	preferencesDialog->stdWriter->setChecked(config->readBoolEntry("stdWriter", false));
	preferencesDialog->stdOpen->setChecked(config->readBoolEntry("stdOpen", false));
	preferencesDialog->stdLast->setChecked(config->readBoolEntry("stdLast", false));
	preferencesDialog->stdStart->setChecked(config->readBoolEntry("stdStart", false));
	preferencesDialog->nothing->setChecked(config->readBoolEntry("nothing", true));
	config->setGroup("Modules");
	preferencesDialog->calc->setChecked(calc->isEnabled());
	preferencesDialog->draw->setChecked(draw->isEnabled());
	preferencesDialog->impress->setChecked(impress->isEnabled());
	preferencesDialog->global->setChecked(global->isEnabled());
	preferencesDialog->math->setChecked(math->isEnabled());
	preferencesDialog->web->setChecked(web->isEnabled());
	preferencesDialog->writer->setChecked(writer->isEnabled());
	preferencesDialog->open->setChecked(file->isEnabled());
	preferencesDialog->recent->setChecked(recent->isEnabled());

	/** Enable/Disable session-management-bug workaround */
	QString fileName = QDir::homeDirPath() + "/.kturnedon";
	QFile file(fileName);
	file.open(IO_WriteOnly);
	QTextStream kTurnedOn(&file);
	QString value;
	if (preferencesDialog->enableQuick->isChecked()){
		value = "true\n";
		if (preferencesDialog->quickStartFix->isChecked()){
			value+="fix";
		}
	}else{
		value = "false\n";
	}
	kTurnedOn << value;
	file.close();
}
/** Call this to start OpenOffice.org with the given parameters */
void OooQs::startInstances(QStringList params){
	if (params.empty()){
		return;
	}
	KProcess* process = new KProcess;
	*process << soffice << params;
	for ( QStringList::Iterator it = params.begin(); it != params.end(); ++it ) {
		KMessageBox::information(this, *it);
	}
	/** the exit Handler only works, if the father ist notified */
	process->start(KProcess::NotifyOnExit,KProcess::NoCommunication);
	connect( process, SIGNAL (processExited(KProcess*)), SLOT (delProcess(KProcess*)));
}

void OooQs::mousePressEvent(QMouseEvent* event){
	if (event->type() == QEvent::MouseButtonDblClick){
		emit doubleClick();
	}
	if (event->type() == QEvent::MouseButtonPress){
		if (event->button() == QMouseEvent::RightButton){
			popupMenu->exec(QCursor::pos());
		}
	}
}
/** Open the last file the user opened using the open-dialog. */
void OooQs::openLast(){
	QString file;
	config->setGroup("General options");
	file = config->readEntry("Last file");
	if (file != ""){
		startInstances(file);
	}else{
		fileOpen();
	}
}
/** Set the current DoubleClickAction */
void OooQs::setDoubleClickAction(){
	disconnect(SIGNAL (doubleClick()));
	QToolTip::remove(this);
	config->setGroup("Additional");
	if (config->readBoolEntry("stdCalc")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startCalc()));
		QToolTip::add(this, calc->text());
		return;
	}
	if (config->readBoolEntry("stdDraw")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startDraw()));
		QToolTip::add(this, draw->text());
		return;
	}
	if (config->readBoolEntry("stdImpress")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startImpress()));
		QToolTip::add(this, impress->text());
		return;
	}
	if (config->readBoolEntry("stdGlobal")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startGlobal()));
		QToolTip::add(this, global->text());
		return;
	}
	if (config->readBoolEntry("stdMath")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startMath()));
		QToolTip::add(this, math->text());
		return;
	}
	if (config->readBoolEntry("stdWeb")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startWeb()));
		QToolTip::add(this, web->text());
		return;
	}
	if (config->readBoolEntry("stdWriter")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startWriter()));
		QToolTip::add(this, writer->text());
		return;
	}
	if (config->readBoolEntry("stdOpen")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (fileOpen()));
		QToolTip::add(this, file->text());
		return;
	}
	if (config->readBoolEntry("stdLast")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (openLast()));
		config->setGroup("General options");
		if (config->readEntry("Last file") != ""){
			QToolTip::add(this, i18n("Open") + " " + config->readEntry("Last file"));
		}else{
			QToolTip::add(this, "<" + i18n("History empty") + ">");
		}
		return;
	}
	if (config->readBoolEntry("stdStart")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (openStart()));
		QToolTip::add(this, start->text());
		return;
	}
	QToolTip::add(this, suite + " " + i18n("Quickstarter"));
}
/** Call this to clear the list of recent files. */
void OooQs::clearURLList(){
	QString key;
	/** clear old entries */
	config->setGroup("General options");
	config->writeEntry("Last file", "");
	/** thanks to David Faure for this code */
	config->setGroup("RecentFiles");
	for( unsigned int i = 1 ; i <= recent->maxItems() ; i++ )
	{
		key = QString( "File%1" ).arg( i );
		config->writeEntry( key, QString::null );
	}

	config->sync();
	recent->clearURLList();
}

/** Kills the OpenOffice.org Quickstart-Instance. */
void OooQs::killQuickstart(){
	if (instance.isRunning())
		instance.kill();
}
/** Return the actionCollection. */
KActionCollection* OooQs::getActionCollection(){
	return actionCollection;
}
/** Set the appnameLabels. */
void OooQs::setupLabels(bool complete){
	KConfig* kickerconf = new KConfig("kickerrc", true);
	/** Read from the config if we use OpenOffice.org or Star Office
			and set the suite- and IconName-strings accordingly */
	if (complete){
		KGlobal::config()->setGroup("General options");
		if (KGlobal::config()->readBoolEntry("suite", true)){
			suite = i18n("OpenOffice.org");
			calcIconName="ooo_calc";
			drawIconName="ooo_draw";
			globalIconName="ooo_global";
			impressIconName="ooo_impress";
			mathIconName="ooo_math";
			webIconName="ooo_web";
			writerIconName="ooo_writer";
			adminIconName="ooo_printeradmin";
		}else{
			suite = i18n("Star Office");
			calcIconName="004_spreadsheet_document";
			drawIconName="006_drawing_document";
			globalIconName="011_global_document";
			impressIconName="008_presentation_document";
			mathIconName="015_math_document";
			webIconName="012_html_document";
			writerIconName="002_text_document";
			adminIconName="501_printeradmin";
		}
	}
	/** Build the labels on the base of "suite" */
	calcLabel = suite + " ";
	drawLabel = calcLabel;
	globalLabel = calcLabel;
	impressLabel = calcLabel;
	mathLabel = calcLabel;
	webLabel = calcLabel;
	writerLabel = calcLabel;
	calcLabel+=i18n("Calc");
	drawLabel+=i18n("Draw");
	globalLabel+=i18n("Global");
	impressLabel+=i18n("Impress");
	mathLabel+=i18n("Math");
	webLabel+=i18n("Web");
	writerLabel+=i18n("Writer");
	/** Read from the config if we use detailed menus */
	kickerconf->setGroup("menus");
	if (KGlobal::config()->readBoolEntry("detailed", kickerconf->readBoolEntry("DetailedMenuEntries"))){
		/** Construct the details */
		calcLabel+=" (";
		calcLabel+=i18n("Spreadsheet");
		calcLabel+=")";
		drawLabel+=" (";
		drawLabel+=i18n("Drawing");
		drawLabel+=")";
		globalLabel+=" (";
		globalLabel+=i18n("All types");
		globalLabel+=")";
		impressLabel+=" (";
		impressLabel+=i18n("Presentation");
		impressLabel+=")";
		mathLabel+=" (";
		mathLabel+=i18n("Formular editor");
		mathLabel+=")";
		webLabel+=" (";
		webLabel+=i18n("Webpage editor");
		webLabel+=")";
		writerLabel+=" (";
		writerLabel+=i18n("Textprocessor");
		writerLabel+=")";
	}
	if (complete){
		updateActions();
	}
	preferencesDialog->updateStrings();
}
/** Set the texts on the actions opening the connected module. */
void OooQs::updateActions(void){
	calc->setText(calcLabel);
	calc->setIcon(calcIconName);
	draw->setText(drawLabel);
	draw->setIcon(drawIconName);
	global->setText(globalLabel);
	global->setIcon(globalIconName);
	impress->setText(impressLabel);
	impress->setIcon(impressIconName);
	math->setText(mathLabel);
	math->setIcon(mathIconName);
	web->setText(webLabel);
	web->setIcon(webIconName);
	writer->setText(writerLabel);
	writer->setIcon(writerIconName);
	admin->setText(i18n("Start %1 printer administration").arg(suite));
	admin->setIcon(adminIconName);
}
/** Asks the user if he wants to start spadmin as root or as the current user. */
void OooQs::startSPAdmin(){
	KProcess* process = new KProcess;
	if (KMessageBox::questionYesNo(this, i18n("%1 printer administration needs to be started as root if you have installed %1 with the network-option. If you start it as a usual user, you won't be able to save your changed settings.\n\nDo you want to start it as root? (You'll need root's password)").arg(suite).arg(suite)) == 3){
		*process << "kdesu";
	}
	KGlobal::config()->setGroup("General options");
	*process << KGlobal::config()->readEntry("exec") + "/spadmin";
	process->start(KProcess::NotifyOnExit,KProcess::NoCommunication);
}
/** Returns soffice. */
QString OooQs::getSoffice(void){
	return soffice;
}
