/***************************************************************************
                          opendialog.cpp  -  description
                             -------------------
    begin                : Sun May 19 2002
    copyright            : (C) 2002 by Christian Nitschkowski
    email                : segfault_ii@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "opendialog.h"
#include <oooqs.h>

#include <qheader.h>
#include <qvbox.h>
#include <qwidget.h>
#include <qwhatsthis.h>
#include <kiconloader.h>
#include <klocale.h>
#include <ktoolbar.h>
#include <kmessagebox.h>

#define Icon(x) KGlobal::instance()->iconLoader()->loadIcon(x, KIcon::NoGroup)
#define SmallIcon(x) KGlobal::instance()->iconLoader()->loadIcon(x, KIcon::NoGroup, KIcon::SizeSmall)

openDialog::openDialog(QWidget *parent, const char *name, QString title ) : KDialogBase(KDialogBase::IconList,
					title, KDialogBase::Help | KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Cancel, parent, name)
{
	QVBox* newDoc = addVBoxPage( i18n("New Document"), i18n("Create a new document"), Icon("filenew"));
	newDocView = new KListView(newDoc);
	newDocEmpty = new QLabel(i18n("No %1-componentes activated in the configuration dialog").arg(((OooQs*)parent)->suite), newDoc);
	QVBox* wizard = addVBoxPage( i18n("Wizards"), i18n("Start with a Wizard"), Icon("wizard"));
	wizardView = new KListView(wizard);
	wizardEmpty = new QLabel(i18n("No wizards activated.\nActivate \"%1 Writer\" in the configuration dialog").arg(((OooQs*)parent)->suite), wizard);
	QVBox* templates = addVBoxPage( i18n("Templates"), i18n("Create a new document from template"), Icon("oooqs"));

	/** Create the toolbar */
	KToolBar* toolbar = new KToolBar(templates, 0, true);

	templateView = new KDirOperator("", templates, 0);
	templatesEmpty = new QLabel(i18n("No templates-directory selected in the configuration dialog"), templates);
	templateView->setOnlyDoubleClickSelectsFiles(true);
	templateView->setView(KFile::Simple);
	templateView->view()->setViewMode(KFileView::Files);
	templateView->setNameFilter("*.stw *.stc *.std *.sti");
	templateView->setMode(KFile::Files);

	/** Fill the toolbar */
	toolbar->setIconText(KToolBar::IconOnly);
	toolbar->setBarPos(KToolBar::Top);
	toolbar->enableMoving(false);
	toolbar->setFlat(true);
	
	templateView->actionCollection()->action("up")->plug(toolbar);
	templateView->actionCollection()->action("back")->plug(toolbar);
	templateView->actionCollection()->action("forward")->plug(toolbar);
	templateView->actionCollection()->action("reload")->plug(toolbar);
	templateView->actionCollection()->action("separator")->plug(toolbar);

	KActionMenu* menu = new KActionMenu( i18n("Extras"), "misc", this);
	menu->insert(templateView->actionCollection()->action("sorting menu"));
	menu->insert(templateView->actionCollection()->action("view menu"));
	menu->insert(templateView->actionCollection()->action("separator"));
	menu->insert(templateView->actionCollection()->action("show hidden"));
	menu->insert(templateView->actionCollection()->action("separator"));
	menu->insert(templateView->actionCollection()->action("properties"));
	menu->setDelayed(false);
	connect (menu->popupMenu(), SIGNAL(aboutToShow()), templateView, SLOT(updateSelectionDependentActions()));

	menu->plug(toolbar);
	templateView->actionCollection()->action("separator")->plug(toolbar);
	templateView->actionCollection()->action("short view")->plug(toolbar);
	templateView->actionCollection()->action("detailed view")->plug(toolbar);

	toolbar->setFrameShadow(QFrame::Plain);
	toolbar->adjustSize();

	QWhatsThis::add(templateView, i18n("Select a template you want to start with."));

	/** Set up the newDocView */
	newDocView->addColumn("");
	newDocView->header()->hide();
	newDocView->setResizeMode(QListView::AllColumns);
	newDocView->setSelectionMode(QListView::Single);

	/** Set up the wizardView */
	wizardView->addColumn("");
	wizardView->header()->hide();
	wizardView->setResizeMode(QListView::AllColumns);
	wizardView->setSelectionMode(QListView::Single);

	/** Set the standard-selections */
	KConfig* config = KGlobal::config();
	QString selected;


	/** Set the standard-page */
	config->setGroup("General options");
	if (config->readEntry("save")){
		config->setGroup("Start_with");
		selected = config->readEntry("selected");
		if (selected == "newDoc"){
			showPage(0);
		}else if (selected == "wizard"){
			showPage(1);
		}else if (selected == "templates"){
			showPage(2);
		}
	}else{
		showPage(0);
	}

	/** Make the connections */
	connect (newDocView, SIGNAL (doubleClicked(QListViewItem*)), this, SLOT (accept()));
	connect (wizardView, SIGNAL (doubleClicked(QListViewItem*)), this, SLOT (accept()));
	connect (templateView, SIGNAL (fileSelected(const KFileItem*)), this, SLOT (fileSelected()));
	connect (this, SIGNAL (aboutToShowPage(QWidget*)), this, SLOT (changedPage(QWidget*)));
}
openDialog::~openDialog(){
}
/** Fill the newDocView */
void openDialog::setup_newDocView(int entries){
	/** Reset the contents */
	QWhatsThis::add(newDocView, i18n("Select a type of a document. After selecting a document-type and clicking on \"%1\" a new instance of %2 will be opened with the select document-type. You can double click on the document-type as well to open a new document.").arg(i18n("OK")).arg(((OooQs*)parent())->suite));
	newDocView->clear();
	if (entries == NoneM){
		newDocView->hide();
		newDocEmpty->show();
		return;
	}
	newDocView->show();
	newDocEmpty->hide();
	if ((entries&Calc) == Calc){
		calc = new KListViewItem(newDocView);
		calc->setText(0, ((OooQs*)parent())->calcLabel);
		calc->setPixmap(0, SmallIcon(((OooQs*)parent())->calcIconName));
	}
	if ((entries&Draw) == Draw){
		draw = new KListViewItem(newDocView);
		draw->setText(0, ((OooQs*)parent())->drawLabel);
		draw->setPixmap(0, SmallIcon(((OooQs*)parent())->drawIconName));
	}
	if ((entries&Impress) == Impress){
		impress = new KListViewItem(newDocView);
		impress->setText(0, ((OooQs*)parent())->impressLabel);
		impress->setPixmap(0, SmallIcon(((OooQs*)parent())->impressIconName));
	}
	if ((entries&Global) == Global){
		global = new KListViewItem(newDocView);
		global->setText(0, ((OooQs*)parent())->globalLabel);
		global->setPixmap(0, SmallIcon(((OooQs*)parent())->globalIconName));
	}
	if ((entries&Math) == Math){
		math = new KListViewItem(newDocView);
		math->setText(0, ((OooQs*)parent())->mathLabel);
		math->setPixmap(0, SmallIcon(((OooQs*)parent())->mathIconName));
	}
	if ((entries&Web) == Web){
		web = new KListViewItem(newDocView);
		web->setText(0, ((OooQs*)parent())->webLabel);
		web->setPixmap(0, SmallIcon(((OooQs*)parent())->webIconName));
	}
	if ((entries&Writer) == Writer){
		writer = new KListViewItem(newDocView);
		writer->setText(0, ((OooQs*)parent())->writerLabel);
		writer->setPixmap(0, SmallIcon(((OooQs*)parent())->writerIconName));
	}

	/** Set the standard-selection in newDoc */
	KConfig* config = KGlobal::config();
	config->setGroup("General options");
	if (!config->readBoolEntry("save")){
		if (newDocView->firstChild())
			newDocView->setSelected(newDocView->firstChild(), true);
		return;
	}
	QString selected;

	config->setGroup("Start_with");
	selected = config->readEntry("newDoc_selected", "calc");
	if (selected == "calc"){
		newDocView->setSelected(calc, true);
	}else if (selected == "draw"){
		newDocView->setSelected(draw, true);
	}else if (selected == "impress"){
		newDocView->setSelected(impress, true);
	}else if (selected == "global"){
		newDocView->setSelected(global, true);
	}else if (selected == "math"){
		newDocView->setSelected(math, true);
	}else if (selected == "web"){
		newDocView->setSelected(web, true);
	}else if (selected == "writer"){
		newDocView->setSelected(writer, true);
	}
}
/** Fill the wizardView */
void openDialog::setup_wizardView(int entries){
	/** Reset the contents */
	QWhatsThis::add(wizardView, i18n("Select a wizard. %1 will be started with the selected wizard.").arg(((OooQs*)parent())->suite));
	wizardView->clear();
	if (entries == NoneM){
		wizardView->hide();
		wizardEmpty->show();
		return;
	}
	wizardView->show();
	wizardEmpty->hide();
	if ((entries&Agenda) == Agenda){
		agenda = new KListViewItem(wizardView);
		agenda->setText(0, i18n("Agenda"));
		agenda->setPixmap(0, SmallIcon("text_wizard"));
	}
	if ((entries&Letter) == Letter){
		letter = new KListViewItem(wizardView);
		letter->setText(0, i18n("Letter"));
		letter->setPixmap(0, SmallIcon("kword"));
	}
	if ((entries&Fax) == Fax){
		fax = new KListViewItem(wizardView);
		fax->setText(0, i18n("Fax"));
		fax->setPixmap(0, SmallIcon("kfax"));
	}
	if ((entries&Label) == Label){
		label = new KListViewItem(wizardView);
		label->setText(0, i18n("Label"));
		label->setPixmap(0, SmallIcon("info"));
	}
	if ((entries&Memo) == Memo){
		memo = new KListViewItem(wizardView);
		memo->setText(0, i18n("Memo"));
		memo->setPixmap(0, SmallIcon("knode"));
	}
	if ((entries&VCARD) == VCARD){
		vcard = new KListViewItem(wizardView);
		vcard->setText(0, i18n("VCARD"));
		vcard->setPixmap(0, SmallIcon("vcard"));
	}

	/** Set the standard-selection in wizard */
	KConfig* config = KGlobal::config();
	config->setGroup("General options");
	if (!config->readBoolEntry("save")){
		if (wizardView->firstChild())
			wizardView->setSelected(wizardView->firstChild(), true);
		return;
	}
	QString selected;

	config->setGroup("Start_with");
	selected = config->readEntry("wizard_selected");
	if (selected == "agenda"){
		wizardView->setSelected(agenda, true);
	}else if (selected == "fax"){
		wizardView->setSelected(fax, true);
	}else if (selected == "label"){
		wizardView->setSelected(label, true);
	}else if (selected == "letter"){
		wizardView->setSelected(letter, true);
	}else if (selected == "memo"){
		wizardView->setSelected(memo, true);
	}else if (selected == "vcard"){
		wizardView->setSelected(vcard, true);
	}
}
/** Called when the user clicks on the OK-Button. */
void openDialog::accept(){
	QStringList command;
	if (activePageIndex() == pageIndex((QVBox*)newDocView->parent())){
		if (newDocView->selectedItem() == calc){
			command = "private:factory/scalc";
		}
		if (newDocView->selectedItem() == draw){
			command = "private:factory/sdraw";
		}
		if (newDocView->selectedItem() == impress){
			command = "private:factory/simpress";
		}
		if (newDocView->selectedItem() == global){
			command = "private:factory/swriter/GlobalDocument";
		}
		if (newDocView->selectedItem() == math){
			command = "private:factory/smath";
		}
		if (newDocView->selectedItem() == web){
			command = "private:factory/swriter/web";
		}
		if (newDocView->selectedItem() == writer){
			command = "private:factory/swriter";
		}
	}
	if (activePageIndex() == pageIndex((QVBox*)wizardView->parent())){
		if (wizardView->selectedItem() == agenda){
			command = "slot:10426";
		}
		if (wizardView->selectedItem() == letter){
			command = "slot:10428";
		}
		if (wizardView->selectedItem() == fax){
			command = "slot:10427";
		}
		if (wizardView->selectedItem() == label){
			command = "private:factory/swriter?slot=21051";
		}
		if (wizardView->selectedItem() == memo){
			command = "slot:10429";
		}
		if (wizardView->selectedItem() == vcard){
			command = "private:factory/swriter?slot=21052";
		}
	}
	if (activePageIndex() == pageIndex((QVBox*)templateView->parent())){
		const KFileItemList* files = templateView->selectedItems();
		for (KFileItemListIterator it(*files); it; ++it){
			command+= it.current()->url().url();
		}
	}
	if (command.empty()){
		KMessageBox::sorry(this, i18n("You have to select a document-type, a wizard or at least one template first."));
	}else{
		open(command);
	}
}
/** Should be called when the user selected a file in the templateView. */
void openDialog::fileSelected(void){
	QString selectedFiles;
	const KFileItemList* files = templateView->selectedItems();
	for (KFileItemListIterator it(*files); it; ++it){
		selectedFiles+= it.current()->url().url();
	}
	open(selectedFiles);
}
/** Called when the user selected the module or template to start with. */
void openDialog::open(QStringList command){
	QString page, selectedItem;
	KConfig* config = KGlobal::config();

	emit selected(command);
	config->setGroup("General options");
	if (config->readEntry("save")){
		config->setGroup("Start_with");
		/** Test every possibility to be absolutely sure the correct page and selected items are saved. */
		if (activePageIndex() == pageIndex((QVBox*)newDocView->parent())){
			page = "newDoc";
		}
		if (activePageIndex() == pageIndex((QVBox*)wizardView->parent())){
			page = "wizard";
		}
		if (activePageIndex() == pageIndex((QVBox*)templateView->parent())){
			page = "templates";
		}
		config->writeEntry("selected", page);
		/** Set standard-value */
		selectedItem = "calc";
		/** Check which item is selected */
		if (newDocView->selectedItem() == calc){
			selectedItem = "calc";
		}
		if (newDocView->selectedItem() == draw){
			selectedItem = "draw";
		}
		if (newDocView->selectedItem() == impress){
			selectedItem = "impress";
		}
		if (newDocView->selectedItem() == global){
			selectedItem = "global";
		}
		if (newDocView->selectedItem() == math){
			selectedItem = "math";
		}
		if (newDocView->selectedItem() == web){
			selectedItem = "web";
		}
		if (newDocView->selectedItem() == writer){
			selectedItem = "writer";
		}
		config->writeEntry("newDoc_selected", selectedItem);
		/** Set new standard-value */
		selectedItem = "agenda";
		/** Check which item is selected */
		if (wizardView->selectedItem() == agenda){
			selectedItem = "agenda";
		}
		if (wizardView->selectedItem() == fax){
			selectedItem = "fax";
		}
		if (wizardView->selectedItem() == label){
			selectedItem = "label";
		}
		if (wizardView->selectedItem() == letter){
			selectedItem = "letter";
		}
		if (wizardView->selectedItem() == memo){
			selectedItem = "memo";
		}
		if (wizardView->selectedItem() == vcard){
			selectedItem = "vcard";
		}
		config->writeEntry("wizard_selected", selectedItem);

		/** Commented out because of no use at the moment */
		/** Reset the contents of selected to fill it with the selected files */
		selectedItem = "";
		const KFileItemList* files = templateView->selectedItems();
		for (KFileItemListIterator it(*files); it; ++it){
			selectedItem+= it.current()->url().prettyURL();
			if (!it.atLast()){
				selectedItem+=",";
			}
		}
		config->writeEntry("templates_selected", selectedItem);

		config->sync();
	}
	close();
}
/** Set some properties of the diroperator and the corresponding file view. */
void openDialog::setup_templateView(void){
	KGlobal::config()->setGroup("General options");
	if (KGlobal::config()->readEntry("templates") == "" ){
		templatesEmpty->show();
		templateView->hide();
	}else{
		const KURL templateDir = KGlobal::config()->readEntry("templates");
		templateView->setURL(templateDir, true);
		templateView->updateDir();
		templatesEmpty->hide();
		templateView->show();
#ifdef DEBUG
		/** Not working - not planned for 1.0
			Should select a list of files, read from KGlobal::config */
		templateView->updateDir();
		templateView->clearHistory();
		if (KGlobal::config()->readBoolEntry("save")){
			QString current;
			templateView->view()->updateView(true);
			KGlobal::config()->setGroup("Start_with");
			const KFileItemList* files = templateView->view()->items();
			for (int i=0; i >= 0; i++){
				current = KGlobal::config()->readEntry("templates_selected").section(',', i, i);
				if (current == ""){
					break;
				}
				for (KFileItemListIterator it(*files); it.current(); ++it){
					if (current == it.current()->url().prettyURL()){
						templateView->view()->setSelected(it.current(), true);
					}
				}
			}
			templateView->view()->updateView(true);
		}
#endif
	}
}
/** Called when a new page is about to be shown. */
void openDialog::changedPage(QWidget* newPage){
	setHelp("start-with");
}
/** Checks if the selected entries can be selected and takes action if they don't. */
void openDialog::checkStates(void){
	if (((newDocView->isHidden()) && (wizardView->isHidden())) && (templateView->isHidden())){
		enableButtonOK(false);
	}else{
		enableButtonOK(true);
	}
}
