/***************************************************************************
                          opendialog.h  -  description
                             -------------------
    begin                : Sun May 19 2002
    copyright            : (C) 2002 by Christian Nitschkowski
    email                : segfault_ii@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef OPENDIALOG_H
#define OPENDIALOG_H

#include <kdialogbase.h>
#include <kdiroperator.h>
#include <klistview.h>

/**The dialog from where you can start OOo-instances, load your documents and call OOo-Autopilots.
  *@author Christian Nitschkowski
  */

enum OOoModules {NoneM=0, Calc = 1, Draw = 2, Impress = 4, Global = 8, Math = 16, Web = 32, Writer = 64, AllM = 255 };
enum OOoWizards {NoneW=0, Agenda = 1, Letter = 2, Fax = 4, Label =8, Memo = 16, VCARD = 32, AllW = 255 };

class openDialog : public KDialogBase  {
	Q_OBJECT
	friend class OooQs;
public:
	openDialog(QWidget *parent=0, const char *name=0, QString title="");
	~openDialog();
private: // Private attributes
	/** The list containing the different OOo-modules. */
	KListView* newDocView;
	/** The list containing the different OOo-wizards/autopilots. */
	KListView* wizardView;
	/** The list containing the different OOo-templates. */
	KDirOperator* templateView;
	/** Show the user that he has disabled all modules */
	QLabel* newDocEmpty;
	/** Show the user that he has disabled all wizards */
	QLabel* wizardEmpty;
	/** Show the user that he has disabled the templates */
	QLabel* templatesEmpty;
	/** The OpenOffice.org Calc item. */
	KListViewItem* calc;
	/** The OpenOffice.org Draw item. */
	KListViewItem* draw;
	/** The OpenOffice.org Impress item. */
	KListViewItem* impress;
	/** The OpenOffice.org Global item. */
	KListViewItem* global;
	/** The OpenOffice.org Math item. */
	KListViewItem* math;
	/** The OpenOffice.org Web item. */
	KListViewItem* web;
	/** The OpenOffice.org Writer item. */
	KListViewItem* writer;
	/** The OpenOffice.org Agenda item. */
	KListViewItem* agenda;
	/** The OpenOffice.org Letter item. */
	KListViewItem* letter;
	/** The OpenOffice.org Fax item. */
	KListViewItem* fax;
	/** The OpenOffice.org Memo item. */
	KListViewItem* label;
	/** The OpenOffice.org Memo item. */
	KListViewItem* memo;
	/** The OpenOffice.org VCARD item. */
	KListViewItem* vcard;
public slots: // Public slots
	/** Fill the newDocView */
	virtual void setup_newDocView(int entries=NoneM);
	/** Fill the wizardView */
	virtual void setup_wizardView(int entries=NoneW);
	/** Called when the user clicks on the OK-Button. */
	virtual void accept();
	/** Checks if the selected entries can be selected and takes action if they don't. */
	void checkStates(void);
	/** Set some properties of the diroperator and the corresponding file view. */
	void setup_templateView(void);
signals: // Signals
	/** Will be emitted when the user double-clicks on an entry or on the OK-button. */
	void selected(QStringList);
private slots: // Private slots
	/** Should be called when the user selected a file in the templateView. */
	virtual void fileSelected(void);
	/** Called when the user selected the module or template to start with. */
	virtual void open(QStringList);
	/** Called when a new page is about to be shown. */
	void changedPage(QWidget* newPage);
};

#endif
